#define _POSIX_C_SOURCE 200809L

#include <lektor/database.h>
#include <lektor/macro.h>

#include <limits.h>
#include <stdio.h>
#include <string.h>

bool
database_search_queue_init(sqlite3 *db, char *col_name, char *rgx, struct lkt_search *ret)
{
    if (ret == NULL) {
        fprintf(stderr, " ! database_search_init: Exit because return pointer is NULL\n");
        return false;
    }

    static const char *SQL_STMT_TEMPLATE =
        "WITH content AS ("
        " SELECT kara.id AS id, string AS any_col, LENGTH(CAST(kara.id AS TEXT)) AS len"
        " FROM kara WHERE %s LIKE ?)"
        "SELECT id, any_col, (SELECT MAX(len) FROM content)"
        "FROM content LIMIT %d OFFSET %d;";
    char SQL_STMT[LKT_MAX_SQLITE_STATEMENT];

    snprintf(SQL_STMT, LKT_MAX_SQLITE_STATEMENT - 1, SQL_STMT_TEMPLATE, col_name,
             ret->msg_count, ret->continuation);
    SQL_STMT[LKT_MAX_SQLITE_STATEMENT - 1] = 0;
    SQLITE_PREPARE(db, ret->stmt, SQL_STMT, error);
    SQLITE_BIND_TEXT(db, ret->stmt, 1, rgx, error);
    ret->db = db;
    /* Assign the callback. */
    return true;
error:
    sqlite3_finalize(ret->stmt);
    return false;
}

bool
database_search_iter(struct lkt_search *item)
{
    const char *sql_row;
    int id, code, id_len;

    code = sqlite3_step(item->stmt);

    if (item == NULL) {
        fprintf(stderr, " * database_search_iter: Exit function because the sqlite3_stmt* is NULL\n");
        return false;
    }

    if (code == SQLITE_DONE)
        goto error_or_done;

    if (code != SQLITE_ROW)
        goto error;

    switch (item->type) {
    case lkt_search_database:
        id = sqlite3_column_int(item->stmt, 0);
        sql_row = (const char *) sqlite3_column_text(item->stmt, 1);
        id_len = sqlite3_column_int(item->stmt, 2);
        return ((lkt_search_database_func) item->call)(item->srv, item->c, id, id_len, sql_row);
    case lkt_search_queue:
    case lkt_search_playlist:
    default:
        goto error;
    }

error:
    fprintf(stderr, " ! database_search_iter: sqlite3_step failed: %s\n", sqlite3_errmsg(item->db));
error_or_done:
    sqlite3_finalize(item->stmt);
    return false;
}
