#define _POSIX_C_SOURCE 200809L

#include <lektor/database.h>
#include <lektor/macro.h>
#include <stdio.h>

bool
database_user_authentificate(sqlite3 *db, const char *password)
{
    static const char *SQL_STMT = "SELECT username FROM users WHERE password = ?";
    sqlite3_stmt *stmt = 0;
    bool ret = false;

    SQLITE_PREPARE(db, stmt, SQL_STMT, error);
    SQLITE_BIND_TEXT(db, stmt, 1, password, error);

    if (sqlite3_step(stmt) != SQLITE_ROW) {
        fprintf(stderr, " ! database_user_authentificate: No user found\n");
        goto error;
    }

    fprintf(stderr, " * User authentification was successfull for user %s\n",
            sqlite3_column_text(stmt, 0));

    ret = true;
error:
    sqlite3_finalize(stmt);
    return ret;
}

bool
database_user_add(sqlite3 *db, const char *username, const char *password)
{
    static const char *SQL_STMT = "INSERT INTO users (username, password) VALUES (?, ?)";
    sqlite3_stmt *stmt = 0;
    bool ret = false;

    SQLITE_PREPARE(db, stmt, SQL_STMT, error);
    SQLITE_BIND_TEXT(db, stmt, 1, username, error);
    SQLITE_BIND_TEXT(db, stmt, 2, password, error);

    if (sqlite3_step(stmt) != SQLITE_OK) {
        fprintf(stderr, " ! database_user_add: Failed to add user %s: %s\n",
                username, sqlite3_errmsg(db));
        goto error;
    }

    fprintf(stderr, " * User %s added successfully\n", username);

    ret = true;
error:
    sqlite3_finalize(stmt);
    return ret;
}
